/*Reglage et surveillance ampli 2.4GHz par F6BQP   V1.3 le 08/03/2025
  pour PA SP2JYR avec BLC10G22XS-600AVT et controleur de BIAS LMP 92066
  Prevu pour fonctionner avec ESP32 + afficheur TFT 480 X 320 ILI9488
*/
// The SPIFFS (FLASH filing system) is used to hold touch screen calibration data
#include "FS.h"
#include <SPI.h>
#include <TFT_eSPI.h>
#include <Wire.h>             // Librairie IIC
#include <Preferences.h>      // Librairie pour memoire flash
//#include <esp_task_wdt.h>     // Watchdog
Preferences calibr;           // "calibr" is the name of the Preferences object.

#define T_pepHOLD    400      // msec pep hold time before return

//----------------------- Affectation des Pins ESP32 ----------------------------
#define I2C_SDA 21
#define I2C_SCL 17

#define MODE 5                // choix REGLAGE / SURVEILLANCE
#define transmit 26           // entree PTT
#define ALIM13V_ON 12         // etablit le +13V
#define FAN 13                // Ventilateur
#define BIAS_OFF 14           // coupe la polar
#define ALIM28V_OFF 27        // coupe le +28V
#define test_temps 16         // pour mesure temps de boucle

uint16_t analogPWR = 34;      // initialise  PWR
uint16_t analogV28 = 35;      // initialise  Tension 28
uint16_t analogI28 = 33;      // initialise  Courant 28
uint16_t analogV13 = 32;      // initialise  Tension 13
uint16_t analogI13 = 39;      // initialise  Courant 13

/********   Valeurs theoriques de calibration   **********
uint16_t HALF28 = 1888;   // valeur ADC (28V) pour milieu tension 3.3V
uint16_t HALF13 = 1884;   // valeur ADC (13V) pour milieu tension 3.3V
uint32_t VMAX28 = 3957;   // Tension max en dizaines de mV pour 3.3V sur GPIO 35
uint32_t VMAX13 = 1987;   // Tension max en dizaines de mV pour 3.3V sur GPIO 32
*********************************************************/

uint16_t HALF28;
uint16_t HALF13;
uint32_t VMAX28;
uint32_t VMAX13;

//--------------------------- Constantes -----------------------------------------
const uint16_t TensionMAX28 = 2950;   // seuil alarme tension max en dizaines de mV
const uint16_t TensionMIN28 = 2000;
const uint16_t TensionMAX13 = 1400;   // seuil alarme tension max en dizaines de mV
const uint16_t IntensMAX28 = 14500;   // seuil alarme intensite max en mA

//const byte Tempmax1 = 45;     // seuil alarme temperature Max1 (45)
const byte Tempmax2 = 53;     // seuil alarme temperature Max2 (53)
const byte TempminRx = 33;    // seuil alarme temperature Min  (33)
const byte TempminTx = 40;    // seuil alarme temperature Min  (40)

//---------------------------- Variables -----------------------------------------

uint16_t direct = 1;
uint32_t pwrmW = 1;
uint16_t pwrdBm = 1;
uint16_t oldpwrdBm = 0;
uint16_t pwrdir = 0;          // power forward (watts)
uint16_t pwrdir_max = 1;
uint16_t largeur = 0;
uint16_t LectTension28 = 1;
uint16_t LectTension28old = 0;
uint16_t LectTension13 = 1;
uint16_t LectTension13old = 0;
uint16_t Tension = 0;
uint16_t LectI28 = 1;
uint16_t LectI13 = 1;
uint16_t Intensite = 0;
uint16_t Intensite_max28 = 1;
uint16_t Intensite_max28old = 0;
uint16_t Intensite_max13 = 1;
uint16_t Intensite_max13old = 0;
float afIntensite = 0.0;
float afTension = 0.0;
long lastTemps1 = 0;
long lastTpep = 0;            // update PEP display
long lastTiMAX28 = 0;
long lastTiMAX13 = 0;

byte  secondes = 0;
byte  minutes = 0;

TFT_eSPI tft = TFT_eSPI(); // https://github.com/Bodmer/TFT_eSPI

// This is the file name used to store the screen calibration data
// You can change this to create new calibration files
// The SPIFFS file name must start with "/"
#define CALIBRATION_FILE "/TouchCalData2"

// Set REPEAT_CAL to true instead of false to run calibration
// again, otherwise it will only be done once
// Repeat calibration if you change the screen rotation
#define REPEAT_CAL false

#define LARGEUR 10        // largeur du bouton
#define HAUTEUR 30        // hauteur du bouton
#define POSY1 100         // position verticale du bouton1 coulissant
#define POSY2 195         // position verticale du bouton2 coulissant
#define LIMITE_GAUCHE 5   // extrémité gauche
#define LIMITE_DROITE 455 // extrémité droite

uint16_t x_precedent1 = 40 ; // position horizontale précédemment touchée
uint16_t x_precedent2 = 40 ; // position horizontale précédemment touchée
uint16_t posx1;
uint16_t posx2;
//byte tempstore;
byte I2C_address;         // 0x41 = FINAL  0x3F = DRIVER
byte reg_base;            // 0x68 (PEAK) ou 0x6A (MAIN)
uint16_t dac_base;        // valeur dac @ 24°C
unsigned int dac_base_lsb;
unsigned int dac_base_msb;
int acc_level;
int board_temp;
int Celsius = 1;
int oldCelsius = 0;
float temperature;
byte ADR_LUT = 0x41;
byte DEL = 0x44;

boolean reglage = false;
boolean flag1 = false;
boolean flag2 = false;
boolean flag3 = false;
boolean flag4 = false;
boolean flag5 = false;
boolean flag6 = false;
boolean flag7 = false;
boolean flagFAN = false;
boolean reglage_BIAS = false;
boolean reglage_LUT = false;
boolean sav = false;
boolean calib = false;

// Table LUT et BIAS FINAL standard (BIAS constant vs temperature)
byte table_F[43] = {0x44,0x44,0x44,0x44,0x44,0x44,0x44,0x44,0x44,0x44,0x44,0x45,0x45,0x55,0x45,0x55,0x45,0x55,0x45,0x55,0x45,0x55,0x45,0x55,0x45,0x55,0x45,0x55,0x45,0x55,0x45,0x55,0x45,0x55,0x45,0x55,0x45,0x55,0x45,0x16,0x5E,0x13,0xD8};

// Table LUT et BIAS DRIVER standard (PWR constant vs temperature)
byte table_D[43] = {0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x64,0x55,0x64,0x55,0x64,0x55,0x64,0x55,0x64,0x55,0x64,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x6,0x54,0x5,0x8A};

uint16_t tableI[8] ;

// Création de 19 objets boutons
#define NOMBRE_BOUTONS 19
TFT_eSPI_Button bouton[NOMBRE_BOUTONS];

//TaskHandle_t Task0;              // pour utilisation CORE 0

void IRAM_ATTR Inter_PTT()                   // code interruption en RAM
{                                            // si PTT bascule sur GPIO 26
 if (digitalRead(transmit) == LOW)           // si emission
  {
    delayMicroseconds(1000);                 // attente 1 mS
    digitalWrite(BIAS_OFF, LOW);             // Bias ON

    delayMicroseconds(10000);                // attente 10 mS
    delayMicroseconds(10000);                // attente 10 mS
    digitalWrite(ALIM13V_ON, HIGH);          // +13V ON

    lastTemps1 = millis();
    secondes = 1;
    minutes = 0;
  }
   else                                       // si reception
  { 
     delayMicroseconds(3000);                 // attente 3 mS
     digitalWrite(ALIM13V_ON, LOW);           // coupe le +13V
     delayMicroseconds(3000);                 // attente 3 mS
     digitalWrite(BIAS_OFF, HIGH);            // coupe le BIAS
  }
}

void setup()
{
Serial.begin(115200); // Use serial port

calibr.begin("my-prefs", true); // create and open namespace = my-prefs in RO mode
bool doesExist = calibr.isKey("HALF28p"); // Test for the existence of HALF28p key

if (doesExist == false) {
   // If doesExist is false, we will need to create our
   // namespace key(s) and store a value into them
   // Insert your "first time run" code to create your keys & assign their values below here
  calibr.end(); // Close the namespace in RO mode
  calibr.begin("my-prefs", false); // open namespace = my-prefs in RW mode

  calibr.putUShort("HALF28p", 1887); //  valeurs
  calibr.putUShort("HALF13p", 1916); // theoriques
  calibr.putUInt("VMAX28p", 3979);   //    de
  calibr.putUInt("VMAX13p", 1995);   // calibration 

  calibr.end(); // Close the namespace
  calibr.begin("my-prefs", true); // open namespace = my-prefs in RO mode
 }
delay (500);
   //  If doesExist is true, the key(s) we need have been created before
   //  and so we can access their values :
   HALF28 = calibr.getUShort("HALF28p"); // charge les
   HALF13 = calibr.getUShort("HALF13p"); // valeurs de calibration
   VMAX28 = calibr.getUInt("VMAX28p");   // stockees dans la
   VMAX13 = calibr.getUInt("VMAX13p");   // memoire flash de l'ESP32

  Serial.println("HALF28 HALF13 VMAX28 VMAX13");
  Serial.print(HALF28);
  Serial.print("   ");
  Serial.print(HALF13);
  Serial.print("   ");
  Serial.print(VMAX28);
  Serial.print("   ");
  Serial.println(VMAX13);

  calibr.end(); // Close the namespace

  pinMode (MODE, INPUT_PULLUP);         // entree choix mode Surveillance / Reglage
  pinMode (transmit, INPUT_PULLUP);     // entree PTT
  pinMode (ALIM13V_ON, OUTPUT);         // sortie etablissement alim +13V
  pinMode (FAN, OUTPUT);                // sortie ventilateur
  pinMode (BIAS_OFF, OUTPUT);           // sortie coupure Bias
  pinMode (ALIM28V_OFF, OUTPUT);        // sortie coupure alim +28V
  pinMode (test_temps, OUTPUT);         // sortie Test

  digitalWrite(ALIM13V_ON, LOW);        // +13V OFF
  digitalWrite(FAN, LOW);               // ventilateur OFF
  digitalWrite(BIAS_OFF, HIGH);         // Bias OFF
  digitalWrite(ALIM28V_OFF, LOW);       // +28V ON

  Wire.begin(I2C_SDA, I2C_SCL);  // init IIC bus

//------ autorise ecriture dans LMP92066  FINAL et DRIVER -----------
  I2C_address = 0x3F;     // Adresse IIC LMP DRIVER
  acc_level = set_acc_lvl_L2();
  I2C_address = 0x41;     // Adresse IIC LMP FINAL
  acc_level = set_acc_lvl_L2();

  tft.init();
  tft.setRotation(1);  // portrait: 0 ou 2, paysage: 1 ou 3

  touch_calibrate();  // procédure de calibration de l'écran tactile

  tft.fillScreen(TFT_BLACK); // on efface tout (fond noir)
  tft.fillRect(80, 75, 320, 180, TFT_NAVY); // rectangle largeur 320  hauteur 180
  tft.setFreeFont(&FreeSansOblique12pt7b);   // police de caractère
  tft.setTextSize(2);
  tft.setCursor(90, 145);
  tft.println("Ampli 2.4GHz");
  tft.setCursor(90, 225);
  tft.print(" F6BQP V1.3");
  tft.setTextSize(1);
  delay(1500);

if (digitalRead(MODE) == LOW) // mode REGLAGE
 {reglage = true;
  tft.fillScreen(TFT_BLACK); // on efface tout (fond noir)
  tft.fillRect(0, 240, 150, 80, TFT_CYAN);   // rectangle largeur 150  hauteur 80
  tft.fillRect(165, 240, 150, 80, TFT_CYAN); // rectangle largeur 150  hauteur 80
  tft.fillRect(330, 240, 150, 80, TFT_CYAN); // rectangle largeur 150  hauteur 80

  tft.setFreeFont(&FreeSansBold12pt7b);   // police de caractère
  tft.setTextColor(TFT_BLACK); // Set text color to black
  tft.setCursor(42, 312);
  tft.print("FINAL");
  tft.setCursor(192, 312);
  tft.print("DRIVER");

  // création des 6 boutons du bas
  // 1er bouton: centre positionné à x = 39 et y = 264, largeur 66, hauteur 40
  // contour en noir, remplissage vert, texte en noir, taille de texte 1
  bouton[0].initButton(&tft, 39, 264, 66, 40, TFT_BLACK, TFT_GREEN, TFT_BLACK, "BIAS", 1);
  bouton[0].drawButton();

  // 2ième bouton: centre positionné à x = 111 et y = 264, largeur 66, hauteur 40 
  // contour en noir, remplissage jaune, texte en noir, taille de texte 1
  bouton[1].initButton(&tft, 111, 264, 66, 40, TFT_BLACK, TFT_YELLOW, TFT_BLACK, "LUT", 1);
  bouton[1].drawButton();

  // 3ième bouton: centre positionné à x = 204 et y = 264, largeur 66, hauteur 40 
  // contour en noir, remplissage vert, texte en noir, taille de texte 1
  bouton[2].initButton(&tft, 204, 264, 66, 40, TFT_BLACK, TFT_GREEN, TFT_BLACK, "BIAS", 1);
  bouton[2].drawButton();

  // 4ième bouton: centre positionné à x = 276 et y = 264, largeur 66, hauteur 40 
  // contour en noir, remplissage jaune, texte en noir, taille de texte 1
  bouton[3].initButton(&tft, 276, 264, 66, 40, TFT_BLACK, TFT_YELLOW, TFT_BLACK, "LUT", 1);
  bouton[3].drawButton();

  // 5ième bouton: centre positionné à x = 369 et y = 264, largeur 66, hauteur 40 
  // contour en noir, remplissage rouge, texte en blanc, taille de texte 1
  bouton[4].initButton(&tft, 369, 264, 66, 40, TFT_BLACK, TFT_RED, TFT_WHITE, "SAV", 1);
  bouton[4].drawButton();

  // 6ième bouton: centre positionné à x = 441 et y = 264, largeur 66, hauteur 40 
  // contour en noir, remplissage rouge, texte en blanc, taille de texte 1
  bouton[5].initButton(&tft, 441, 264, 66, 40, TFT_BLACK, TFT_RED, TFT_WHITE, "CAL", 1);
  bouton[5].drawButton();

  delay(500);
 }
else
 {reglage = false; // mode SURVEILLANCE
  tft.fillScreen(TFT_BLACK); // on efface tout (fond noir)
  tft.setFreeFont(&FreeSansOblique12pt7b); // police de caractère
  tft.setCursor(115, 25);
  tft.print("Monitoring PA 13cm         F6BQP");

  tft.drawRect(100, 35, 280, 40, TFT_CYAN);   // rectangle largeur 280  hauteur 40
  for (int i = 0; i < 24; i++)                // 24 lignes verticales
   {
  tft.drawLine(102+12*i, 75, 102+12*i, 85, TFT_CYAN);
   }
  tft.drawRect(100, 95, 280, 40, TFT_CYAN);   // rectangle largeur 280  hauteur 40
  tft.drawRect(100, 155, 280, 40, TFT_CYAN);  // rectangle largeur 280  hauteur 40
  tft.drawRect(100, 215, 280, 40, TFT_CYAN);  // rectangle largeur 280  hauteur 40

  tft.fillRect(5, 280, 80, 40, TFT_DARKCYAN);
  tft.setCursor(25, 310);
  tft.print( "Rx");
  tft.fillRect(340, 280, 130, 40, TFT_DARKCYAN);
  tft.setCursor(355, 310);
  tft.print( "FAN OFF");

  tft.setTextColor(TFT_WHITE, TFT_BLACK); // Set text color to white on black
  tft.setCursor(5, 65);
  tft.print("PWR");
  tft.setCursor(5, 245);
  tft.print("TEMP");

//create a task that executes the Task0code() function, with priority 1 and executed on core 0
//xTaskCreatePinnedToCore(Task0code, "Task0", 10000, NULL, 1, &Task0, 0);
delay(500);
//attachInterrupt(digitalPinToInterrupt(26), Inter_PTT,CHANGE); // init interruption sur GPIO 26
 }

}

//------------------------ boucle principale CORE 1 -------------------------------------
void loop(void)
{  digitalWrite(test_temps, HIGH);
//  esp_task_wdt_reset(); // reset watchdog
  delay(1);               // VERY IMPORTANT for Watchdog Reset to apply
 if (reglage == true)
//-------------------------------- Mode REGLAGE ----------------------------------
 {
  board_temp = get_12bit_reg(0x00);
  temperature = twocomplement_temp(board_temp);
  Celsius = int (temperature);

  if (Celsius != oldCelsius)
   {  oldCelsius = Celsius;
    tft.fillRect(330, 0, 150, 32, TFT_CYAN);  // rectangle largeur 175  hauteur 32
    tft.setTextColor(TFT_BLACK);
    tft.setCursor(335, 22);
    tft.print("Temp ");
    tft.print(Celsius);
    tft.drawCircle(446, 12, 5, TFT_BLACK);
    tft.setCursor(456, 22);
    tft.print("C");
   }
  uint16_t t_x = 0, t_y = 0; // coordonnées touchées par l'utilisateur
  boolean pressed = tft.getTouch(&t_x, &t_y); // vrai si contact avec l'écran

//---------- REGLAGE BIAS ------------
if (reglage_BIAS == true)
{  digitalWrite(BIAS_OFF, LOW);             // Bias ON
if (flag1 == true) 
 {  flag1 = false;
 tft.fillRect(0, 40, 480, 200, TFT_BLACK); // pour effacer le dessin précédent

  dac_base = get_12bit_reg(0x68); // PEAK
  if (dac_base < 900) {dac_base = 900;}
  dac_base = dac_base - 900;
  posx1 = dac_base / 2 + 5;

  dac_base = get_12bit_reg(0x6A); // MAIN
  if (dac_base < 900) {dac_base = 900;}
  else if (dac_base > 1800) {dac_base = 1800;}
  dac_base = dac_base - 900;
  posx2 = dac_base / 2 + 5;

  dessineCurseur1(posx1);
  dessineCurseur2(posx2);

// création des 4 boutons du menu REGLAGE BIAS
tft.setFreeFont(&FreeSansBold12pt7b);   // police de caractère
  // 7ième bouton: centre positionné à x = 470 et y = 75, largeur 20, hauteur 20 
  // contour en blanc, remplissage cyan, texte en noir, taille de texte 1
  bouton[6].initButton(&tft, 470, 77, 20, 20, TFT_WHITE, TFT_CYAN, TFT_BLACK, "+", 1);
  bouton[6].drawButton();

  // 7ième bouton: centre positionné à x = 470 et y = 125, largeur 20, hauteur 20 
  // contour en blanc, remplissage cyan, texte en noir, taille de texte 1
  bouton[7].initButton(&tft, 470, 123, 20, 20, TFT_WHITE, TFT_CYAN, TFT_BLACK, "-", 1);
  bouton[7].drawButton();

  // 7ième bouton: centre positionné à x = 470 et y = 170, largeur 20, hauteur 20 
  // contour en blanc, remplissage cyan, texte en noir, taille de texte 1
  bouton[8].initButton(&tft, 470, 172, 20, 20, TFT_WHITE, TFT_CYAN, TFT_BLACK, "+", 1);
  bouton[8].drawButton();

  // 10ième bouton: centre positionné à x = 470 et y = 220, largeur 20, hauteur 20 
  // contour en blanc, remplissage cyan, texte en noir, taille de texte 1
  bouton[9].initButton(&tft, 470, 218, 20, 20, TFT_WHITE, TFT_CYAN, TFT_BLACK, "-", 1);
  bouton[9].drawButton();

 }
  if (pressed)
 {
   if (t_y > (POSY1 - (HAUTEUR/2)) && t_y < (POSY1 + (HAUTEUR/2)))  // si on touche le curseur1
    {
     if (t_x != x_precedent1 && (t_x < 455))  // si on a changé de position
      {
       dessineCurseur1(t_x);
      }
    }
     else if (t_y > (POSY2 - (HAUTEUR/2)) && t_y < (POSY2 + (HAUTEUR/2))) // si on touche le curseur2
    {
    if (t_x != x_precedent2 && (t_x < 455))  // si on a changé de position
      {
       dessineCurseur2(t_x);
      }
    }
 } 

  // On vérifie si la position du contact correspond à celle d'un bouton
  for (uint8_t numero = 6; numero < 10; numero++) {
    if (pressed && bouton[numero].contains(t_x, t_y)) {
      bouton[numero].press(true);
    } else {
      bouton[numero].press(false);
    }
  }
  // Vérifions maintenant si l'état d'un des 5 boutons a changé
  for (uint8_t numero = 6; numero < 10; numero++)
  {
    // si le bouton vient d'être relâché, on le redessine avec sa forme normale
    if (bouton[numero].justReleased()) {
      bouton[numero].drawButton();
    }

    // si le bouton vient d'être pressé...
    if (bouton[numero].justPressed()) {
      bouton[numero].drawButton(true);  // on le redessine avec les couleurs inversées

      // ...puis on fait ce que l'utilisateur a demandé:

      switch (numero) {

        case 6:    // 7ième bouton ( + )
          dac_base = get_12bit_reg(0x68);
          dac_base ++;
         if (dac_base < 900) {dac_base = 900;}
         else if (dac_base > 1800) {dac_base = 1800;}
          write_dac_base(0x68, dac_base);
          texte_dac_basePeak ();
          break;

        case 7:    // 8ième bouton ( - )
          dac_base = get_12bit_reg(0x68);
          dac_base --;
        if (dac_base < 900) {dac_base = 900;}
        else if (dac_base > 1800) {dac_base = 1800;}
          write_dac_base(0x68, dac_base);
          texte_dac_basePeak ();
          break;

        case 8:    // 9ième bouton ( + )
          dac_base = get_12bit_reg(0x6A);
          dac_base ++;
         if (dac_base < 900) {dac_base = 900;}
         else if (dac_base > 1800) {dac_base = 1800;}
          write_dac_base(0x6A, dac_base);
          texte_dac_baseMain ();
         break;

        case 9:    // 10ième bouton ( - )
          dac_base = get_12bit_reg(0x6A);
          dac_base --;
         if (dac_base < 900) {dac_base = 900;}
         else if (dac_base > 1800) {dac_base = 1800;}
          write_dac_base(0x6A, dac_base);
          texte_dac_baseMain ();
          break;
      }
      delay(10); // anti-rebond
    }
  }
}

//---------- REGLAGE LUT ------------
  else if (reglage_LUT == true)
{  digitalWrite(BIAS_OFF, HIGH);             // Bias OFF
if (flag2 == true)
 {  flag2 = false;
 tft.fillRect(0, 40, 480, 200, TFT_BLACK); // pour effacer le dessin précédent

// dessin des rectangles et 5 boutons
  dessin_rectangles_LUT();
  ADR_LUT = 0x41;
  tft.setCursor(180, 87);
  tft.print(ADR_LUT, HEX);

  DEL = get_register(ADR_LUT);      // lecture DEL dans le LMP
  tft.setCursor(355, 87);
  tft.print(DEL, HEX);

// création des 5 boutons du menu REGLAGE LUT
tft.setFreeFont(&FreeSansBold12pt7b);   // police de caractère
  // 11ième bouton: centre positionné à x = 185 et y = 170, largeur 40, hauteur 40 
  // contour en blanc, remplissage vert, texte en noir, taille de texte 1
  bouton[10].initButton(&tft, 185, 170, 40, 40, TFT_WHITE, TFT_GREEN, TFT_BLACK, "<", 1);
  bouton[10].drawButton();

  // 12ième bouton: centre positionné à x = 295 et y = 170, largeur 40, hauteur 40 
  // contour en blanc, remplissage vert, texte en noir, taille de texte 1
  bouton[11].initButton(&tft, 295, 170, 40, 40, TFT_WHITE, TFT_GREEN, TFT_BLACK, ">", 1);
  bouton[11].drawButton();

  // 13ième bouton: centre positionné à x = 240 et y = 140, largeur 40, hauteur 40 
  // contour en blanc, remplissage vert, texte en noir, taille de texte 1
  bouton[12].initButton(&tft, 240, 140, 40, 40, TFT_WHITE, TFT_GREEN, TFT_BLACK, "+", 1);
  bouton[12].drawButton();

  // 14ième bouton: centre positionné à x = 240 et y = 200, largeur 40, hauteur 40 
  // contour en blanc, remplissage vert, texte en noir, taille de texte 1
  bouton[13].initButton(&tft, 240, 200, 40, 40, TFT_WHITE, TFT_GREEN, TFT_BLACK, "-", 1);
  bouton[13].drawButton();

  // 15ième bouton: centre positionné à x = 432 et y = 170, largeur 80, hauteur 40 
  // contour en blanc, remplissage rouge, texte en noir, taille de texte 1
  bouton[14].initButton(&tft, 432, 170, 80, 60, TFT_WHITE, TFT_RED, TFT_BLACK, "LOAD", 1);
  bouton[14].drawButton();

 } 
  // On vérifie si la position du contact correspond à celle d'un bouton
  for (uint8_t numero = 10; numero < 15; numero++) {
    if (pressed && bouton[numero].contains(t_x, t_y)) {
      bouton[numero].press(true);
    } else {
      bouton[numero].press(false);
    }
  }
  // Vérifions maintenant si l'état d'un des 5 boutons a changé
  for (uint8_t numero = 10; numero < 15; numero++)
  {
    // si le bouton vient d'être relâché, on le redessine avec sa forme normale
    if (bouton[numero].justReleased()) {
      bouton[numero].drawButton();
    }

    // si le bouton vient d'être pressé...
    if (bouton[numero].justPressed()) {
      bouton[numero].drawButton(true);  // on le redessine avec les couleurs inversées

      // ...puis on fait ce que l'utilisateur a demandé:
tft.setFreeFont(&FreeSansOblique12pt7b); // police de caractère
      switch (numero) {

        case 10:    // 11ième bouton ( < )
          --ADR_LUT;                            // decremente adresse LUT
          if (ADR_LUT < 0x41) {ADR_LUT = 0x6B;} // limite a 0x41
          dessin_rectangles_LUT();
          tft.setCursor(180, 87);
          tft.print(ADR_LUT, HEX);

          DEL = get_register(ADR_LUT);      // lecture DEL dans le LMP
          tft.setCursor(355, 87);
          tft.print(DEL, HEX);
          break;

        case 11:    // 12ième bouton ( > )
          ++ADR_LUT;                            // incremente adresse LUT
          if (ADR_LUT > 0x6B) {ADR_LUT = 0x41;} // limite a 0x6B
          dessin_rectangles_LUT();
          tft.setCursor(180, 87);
          tft.print(ADR_LUT, HEX);

          DEL = get_register(ADR_LUT);      // lecture DEL dans le LMP
          tft.setCursor(355, 87);
          tft.print(DEL, HEX);
          break;

        case 12:    // 13ième bouton ( + )
          if (ADR_LUT < 0x68)    
        { ++DEL;                               // incremente valeur DEL
          Wire.beginTransmission(I2C_address); // ecriture DEL dans LMP
          Wire.write(ADR_LUT); // adresse
          Wire.write(DEL); // data
          Wire.endTransmission();
        }
          dessin_rectangles_LUT();
          tft.setCursor(180, 87);
          tft.print(ADR_LUT, HEX);
          tft.setCursor(355, 87);
          tft.print(DEL, HEX);
         break;

        case 13:    // 14ième bouton ( - )
          if (ADR_LUT < 0x68)     
        { --DEL;                               // decremente valeur DEL
          Wire.beginTransmission(I2C_address); // ecriture DEL dans LMP
          Wire.write(ADR_LUT); // adresse
          Wire.write(DEL); // data
          Wire.endTransmission();
        }
          dessin_rectangles_LUT();
          tft.setCursor(180, 87);
          tft.print(ADR_LUT, HEX);
          tft.setCursor(355, 87);
          tft.print(DEL, HEX);
          break;

        case 14:    // 15ième bouton ( LOAD )
         if (I2C_address == 0x41)          // address of LMP92066 FINAL (A0=1 A1=0)
         { for (int i = 0; i <= 42; i++)   // lecture 43 registres
          {
           ADR_LUT = 0x41 + i;   // ADR_LUT 0x41 a 0x6B
           DEL = table_F[i];     // lecture DEL dans table_LUT

           Wire.beginTransmission(I2C_address); // ecriture DEL dans LMP
           Wire.write(ADR_LUT); // adresse
           Wire.write(DEL); // data
           Wire.endTransmission();
          }
         }
         else if ( I2C_address == 0x3F)    // address of LMP92066 DRIVER (A0=0 A1=0)
         { for (int i = 0; i <= 42; i++)   // lecture 43 registres
          {
           ADR_LUT = 0x11 + i;   // ADR_LUT 0x11 a 0x3B
           DEL = table_D[i];     // lecture DEL dans table_LUT

           Wire.beginTransmission(I2C_address); // ecriture DEL dans LMP
           Wire.write(ADR_LUT + 0x30); // adresse
           Wire.write(DEL); // data
           Wire.endTransmission();
          }
         }
          break;
      }
      delay(10); // anti-rebond
    }
  }
}

//---------- SAUVEGARDE ------------
  else if (sav == true)
{  digitalWrite(BIAS_OFF, HIGH);             // Bias OFF
if (flag3 == true)
 {  flag3 = false;
tft.fillRect(0, 40, 480, 200, TFT_BLACK); // pour effacer le dessin précédent
tft.setFreeFont(&FreeSansOblique12pt7b); // police de caractère
  tft.setCursor(1, 70);
  tft.println("Press to write FINAL data in Eeprom");
  tft.print("Burns : ");
  I2C_address = 0x41;     // Adresse IIC LMP FINAL
  tft.print (get_register(0x40));
  tft.print("  Remain : ");
  tft.print (100 - get_register(0x40));

  tft.setCursor(1, 140);
  tft.println("Press to write DRIVER data in Eeprom");
  tft.print("Burns : ");
  I2C_address = 0x3F;     // Adresse IIC LMP DRIVER
  tft.print (get_register(0x40));
  tft.print("  Remain : ");
  tft.print (100 - get_register(0x40));

  tft.setCursor(1, 210);
  tft.println("Press to quit whithout saving");
//  tft.print("Only 100 burns are allowed");


// création des 3 boutons du menu SAUVEGARDE
tft.setFreeFont(&FreeSansBold12pt7b);   // police de caractère

  // 16ième bouton: centre positionné à x = 450 et y = 70, largeur 60, hauteur 50 
  // contour en blanc, remplissage rouge, texte en noir, taille de texte 1
  bouton[15].initButton(&tft, 450, 70, 60, 50, TFT_WHITE, TFT_RED, TFT_BLACK, "SAV", 1);
  bouton[15].drawButton();
 
//else{tft.fillRect(420, 45, 60, 50, TFT_BLACK);} // pour effacer le bouton

  // 17ième bouton: centre positionné à x = 450 et y = 140, largeur 60, hauteur 50 
  // contour en blanc, remplissage rouge, texte en noir, taille de texte 1
  bouton[16].initButton(&tft, 450, 140, 60, 50, TFT_WHITE, TFT_RED, TFT_BLACK, "SAV", 1);
  bouton[16].drawButton();

  // 18ième bouton: centre positionné à x = 450 et y = 210, largeur 60, hauteur 50 
  // contour en blanc, remplissage vert, texte en noir, taille de texte 1
  bouton[17].initButton(&tft, 450, 210, 60, 50, TFT_WHITE, TFT_GREEN, TFT_BLACK, "Quit", 1);
  bouton[17].drawButton();

 }
  // On vérifie si la position du contact correspond à celle d'un bouton
  for (uint8_t numero = 15; numero < 18; numero++) {
    if (pressed && bouton[numero].contains(t_x, t_y)) {
      bouton[numero].press(true);
    } else {
      bouton[numero].press(false);
    }
  }
  // Vérifions maintenant si l'état d'un des 2 boutons a changé
  for (uint8_t numero = 15; numero < 18; numero++)
  {
    // si le bouton vient d'être relâché, on le redessine avec sa forme normale
    if (bouton[numero].justReleased()) {
      bouton[numero].drawButton();
    }

    // si le bouton vient d'être pressé...
    if (bouton[numero].justPressed()) {
      bouton[numero].drawButton(true);  // on le redessine avec les couleurs inversées

      // ...puis on fait ce que l'utilisateur a demandé:

      switch (numero) {

        case 15:    // 16ième bouton ( SAV )

// ------------- ecriture EEPROM LMP92066 FINAL ---------------------------
     I2C_address = 0x41;     // address of LMP92066 FINAL (A0=1 A1=0)
/*       Wire.beginTransmission(I2C_address);
       Wire.write(0x0F); // adresse
       Wire.write(0xE4); // data
       Wire.endTransmission();
*/
         delay (1000); 
         tft.fillRect(0, 45, 480, 60, TFT_BLACK); // pour effacer le dessin précédent
         tft.setCursor(1, 75);
          if (get_register(0x0F) == 0)
           {
            tft.print("DONE   burns: ");
            tft.print (get_register(0x40));
            tft.print("  Remain : ");
            tft.print (100 - get_register(0x40));
           }
          else
           {
            tft.print("Failure");
           }
//         tft.setCursor(150, 200);
//         tft.print("Reset ESP32");
//         while (true) { }           // boucle infinie

          break;

        case 16:    // 17ième bouton ( SAV )

// ------------- ecriture EEPROM LMP92066 DRIVER --------------------------
      I2C_address = 0x3F;     // address of LMP92066 DRIVER (A0=0 A1=0)
/*       Wire.beginTransmission(I2C_address);
       Wire.write(0x0F); // adresse
       Wire.write(0xE4); // data
       Wire.endTransmission();
*/
         delay (1000); 
         tft.fillRect(0, 115, 480, 60, TFT_BLACK); // pour effacer le dessin précédent
         tft.setCursor(1, 145);
          if (get_register(0x0F) == 0)
           {
            tft.print("DONE   burns: ");
            tft.print (get_register(0x40));
            tft.print("  Remain : ");
            tft.print (100 - get_register(0x40));
           }
          else
           {
            tft.print("Failure");
           }
//         tft.setCursor(150, 200);
//         tft.print("Reset ESP32");
//         while (true) { }           // boucle infinie	

          break;

        case 17:    // 18ième bouton ( Quit )
         tft.fillRect(0, 0, 480, 240, TFT_BLACK); // pour effacer le dessin précédent
         sav = false;
          break;

      }
      delay(10); // anti-rebond
    }
  }
}

//---------- CALIBRATION ------------
  else if (calib == true)
{  digitalWrite(BIAS_OFF, HIGH);             // Bias OFF
if (flag7 == true)
 {  flag7 = false;
tft.fillRect(0, 40, 480, 200, TFT_BLACK); // pour effacer le dessin précédent
tft.setFreeFont(&FreeSansOblique12pt7b); // police de caractère
  tft.setCursor(1, 70);
  tft.println("Adjust 28V Power supply to 28.00V");
  tft.println("Adjust 13V Power supply to 13.00V");
  tft.println("Connect anything to outputs 28 & 13V");
  tft.println("Press CAL");

// création du bouton du menu CALIBRATION
tft.setFreeFont(&FreeSansBold12pt7b);   // police de caractère
  // 19ième bouton: centre positionné à x = 432 et y = 195, largeur 60, hauteur 60
  // contour en blanc, remplissage rouge, texte en noir, taille de texte 1
  bouton[18].initButton(&tft, 432, 195, 60, 60, TFT_WHITE, TFT_RED, TFT_BLACK, "CAL", 1);
  bouton[18].drawButton();
 }

  // On vérifie si la position du contact correspond à celle d'un bouton
   uint8_t numero = 18;
    if (pressed && bouton[numero].contains(t_x, t_y)) {
      bouton[numero].press(true);
    } else {
      bouton[numero].press(false);
    }

  // Vérifions maintenant si l'état du bouton a changé
  
    // si le bouton vient d'être relâché, on le redessine avec sa forme normale
    if (bouton[numero].justReleased()) {
      bouton[numero].drawButton();
    }

    // si le bouton vient d'être pressé...
    if (bouton[numero].justPressed()) {
      bouton[numero].drawButton(true);  // on le redessine avec les couleurs inversées

      // ...puis on fait ce que l'utilisateur a demandé:

  calibr.begin("my-prefs", false); // open namespace = my-prefs in RW mode
// Calcul HALF28 moyenne 8 mesures
  for (int i = 0; i < 8; i++)       // lecture 8 valeurs
   {
    tableI[i] = analogRead(analogI28); // 0 a 4095
      delay (10);
   }
    HALF28 = (tableI[0]+tableI[1]+tableI[2]+tableI[3]+tableI[4]+tableI[5]+tableI[6]+tableI[7]) / 8;
    Serial.print(HALF28);
    Serial.print("   ");
  calibr.putUShort("HALF28p", HALF28);

// Calcul HALF13 moyenne 8 mesures
  for (int i = 0; i < 8; i++)       // lecture 8 valeurs
   {
    tableI[i] = analogRead(analogI13); // 0 a 4095
      delay (10);
   }
    HALF13 = (tableI[0]+tableI[1]+tableI[2]+tableI[3]+tableI[4]+tableI[5]+tableI[6]+tableI[7]) / 8;
    HALF13 = HALF13 - 14;        // on retire 14X12= 168mA
    Serial.print(HALF13);
    Serial.print("   ");
  calibr.putUShort("HALF13p", HALF13);

// Calcul VMAX28 moyenne 8 mesures
  for (int i = 0; i < 8; i++)       // lecture 8 valeurs
   {
    tableI[i] = analogRead(analogV28); // 0 a 4095
      delay (10);
   }
    LectTension28 = (tableI[0]+tableI[1]+tableI[2]+tableI[3]+tableI[4]+tableI[5]+tableI[6]+tableI[7]) / 8;
    VMAX28 = 4095 * 2800 / LectTension28 ;  
    Serial.print(VMAX28);
    Serial.print("   ");
  calibr.putUInt("VMAX28p", VMAX28);

// Calcul VMAX13 moyenne 8 mesures
  for (int i = 0; i < 8; i++)       // lecture 8 valeurs
   {
    tableI[i] = analogRead(analogV13); // 0 a 4095
      delay (10);
   }
    LectTension13 = (tableI[0]+tableI[1]+tableI[2]+tableI[3]+tableI[4]+tableI[5]+tableI[6]+tableI[7]) / 8;
    VMAX13 = 4095 * 1300 / LectTension13 ;  
    Serial.println(VMAX13);
  calibr.putUInt("VMAX13p", VMAX13);

delay (2000);

  uint16_t HALF28 = calibr.getUShort("HALF28p");
  uint16_t HALF13 = calibr.getUShort("HALF13p");
  uint32_t VMAX28 = calibr.getUInt("VMAX28p");
  uint32_t VMAX13 = calibr.getUInt("VMAX13p");

  Serial.println("HALF28 HALF13 VMAX28 VMAX13");

  Serial.print(HALF28);
  Serial.print("   ");
  Serial.print(HALF13);
  Serial.print("   ");
  Serial.print(VMAX28);
  Serial.print("   ");
  Serial.println(VMAX13);
  calibr.end(); // Close the namespace

  tft.fillRect(0, 40, 480, 200, TFT_BLACK); // pour effacer le dessin précédent
  tft.setCursor(10, 70);
  tft.println("Calibration done");
  tft.setCursor(10, 150);
  tft.println("Calibration datas are stored");
  tft.print(" in ESP32 Flash memory");

   calib = false;
   }
 }

//-------------- Menu du bas -------------------------------------------
  // On vérifie si la position du contact correspond à celle d'un bouton
  for (uint8_t numero = 0; numero < 6; numero++) {
    if (pressed && bouton[numero].contains(t_x, t_y)) {
      bouton[numero].press(true);
    } else {
      bouton[numero].press(false);
    }
  }

  // Vérifions maintenant si l'état d'un des 5 boutons DU BAS a changé
  for (uint8_t numero = 0; numero < 6; numero++)
  {
    // si le bouton vient d'être relâché, on le redessine avec sa forme normale
    if (bouton[numero].justReleased()) {
      bouton[numero].drawButton();
    }

    // si le bouton vient d'être pressé...
    if (bouton[numero].justPressed()) {
      bouton[numero].drawButton(true);  // on le redessine avec les couleurs inversées

      // ...puis on fait ce que l'utilisateur a demandé:
          tft.setFreeFont(&FreeSansBold12pt7b);   // police de caractère
          tft.setTextColor(TFT_WHITE, TFT_BLACK); // Set text color to white on black
      switch (numero) {
        case 0:    // 1er bouton ( BIAS FINAL )
          tft.fillRect(0, 0, 330, 40, TFT_BLACK); // pour effacer le texte précédent
          tft.setCursor(5, 20);
          tft.print("ADJUST BIAS FINAL");

          reglage_BIAS = true;
          reglage_LUT = false;
          sav = false;
          calib = false;
          flag1 = true;

// à faire ----- SAV BIAS DRIVER et BIAS DRIVER = 0

          I2C_address = 0x41;     // Adresse IIC LMP FINAL
          break;

        case 1:    // 2ième bouton ( LUT FINAL )
          tft.fillRect(0, 0, 330, 40, TFT_BLACK); // pour effacer le texte précédent
          tft.setCursor(5, 20);
          tft.print("ADJUST LUT FINAL");

          reglage_BIAS = false;
          reglage_LUT = true;
          sav = false;
          calib = false;
          flag2 = true;
          I2C_address = 0x41;     // Adresse IIC LMP FINAL
          break;

        case 2:    // 3ième bouton ( BIAS DRIVER )
          tft.fillRect(0, 0, 330, 40, TFT_BLACK); // pour effacer le texte précédent
          tft.setCursor(5, 20);
          tft.print("ADJUST BIAS DRIVER");

          reglage_BIAS = true;
          reglage_LUT = false;
          sav = false;
          calib = false;
          flag1 = true;

// à faire -----  SAV BIAS PEAK et BIAS PEAK = 0

          I2C_address = 0x3F;     // Adresse IIC LMP DRIVER
          break;

        case 3:    // 4ième bouton ( LUT DRIVER )
          tft.fillRect(0, 0, 330, 40, TFT_BLACK); // pour effacer le texte précédent
          tft.setCursor(5, 20);
          tft.print("ADJUST LUT DRIVER");

          reglage_BIAS = false;
          reglage_LUT = true;
          sav = false;
          calib = false;
          flag2 = true;
          I2C_address = 0x3F;     // Adresse IIC LMP DRIVER
          break;

        case 4:    // 5ième bouton ( SAV )
          tft.fillRect(0, 0, 330, 40, TFT_BLACK); // pour effacer le texte précédent
          tft.setCursor(5, 20);
          tft.print("SAV in EEPROM LMP92066");

          reglage_BIAS = false;
          reglage_LUT = false;
          sav = true;
          calib = false;
          flag3 = true;

// à faire -----  RESTORE BIAS PEAK et bias MAIN
          break;

        case 5:    // 6ième bouton ( CAL )
          tft.fillRect(0, 0, 330, 40, TFT_BLACK); // pour effacer le texte précédent
          tft.setCursor(5, 20);
          tft.print("Calibration");

          reglage_BIAS = false;
          reglage_LUT = false;
          sav = false;
          calib = true;
          flag7 = true;
          break;
      }
      delay(10); // anti-rebond
    }
  }
 }

//----------------------------- Mode SURVEILLANCE --------------------------------
 else 
 {
//---------------------------- Affichage Puissance --------------------------------
  direct = analogRead(analogPWR);       // 0 a 4095
  direct = direct + 340;                // ajoute 280mV (seuil diode )
  if (direct < 345) {direct = 0;}       //
  direct = direct / 35;                 // 0 a 126 V sur 50 ohm
  pwrdir = pow(direct,2) / 100;         // calcul puissance directe 0 a 160 W

   if (pwrdir >= pwrdir_max)            // we have a peak !
    {lastTpep = millis();
    pwrdir_max = pwrdir;}               // memorise pwrdir_max

   if (millis() > (lastTpep + T_pepHOLD)) // clear the peak after hold time
    { pwrdir_max = pwrdir; }

  pwrmW = pwrdir_max * 1000;            // puissance en mW 0 a 160000
  pwrdBm = (10 * log10(pwrmW) - 30);    // 0 a + 23  (+30 a +53 dBm)

 if (abs(pwrdBm - oldpwrdBm) >= 1)      // si changement >= 1 dB
{ oldpwrdBm = pwrdBm;
  largeur = pwrdBm * 12;                // 0 a 276

  tft.fillRect(101, 36, 278, 38, TFT_BLACK); // pour effacer le rectangle précédent
  tft.fillRect(101, 36, largeur, 38, TFT_RED);

  tft.fillRect(390, 35, 90, 40, TFT_BLACK); // pour effacer le texte précédent
  tft.setCursor(390, 65);
  tft.print(pwrdir);
  tft.print(" W");
}
digitalWrite(test_temps, LOW);
//---------------------------- Affichage Tension ALIM 28V -------------------------
   LectTension28 = analogRead(analogV28);      // 0 a 4095
   Tension = LectTension28 * VMAX28 / 4095;    // calcul tension 28V en dizaines de mV

 if (Tension > TensionMAX28 )            // si Tension > seuil Alarme
  {
   digitalWrite(ALIM13V_ON, LOW);        // coupe le +13V
   delay(3);
   digitalWrite(BIAS_OFF, HIGH);         // coupe le BIAS
   delay(3);
   digitalWrite(ALIM28V_OFF, HIGH);      // coupe le +28V

   detachInterrupt(digitalPinToInterrupt(26)); // bloque le PTT

   tft.fillScreen(TFT_BLACK);            // efface ecran
   tft.setFreeFont(&FreeSansBold12pt7b); // police de caractère
   tft.setTextColor(TFT_RED, TFT_BLACK); // Set text color to red on black
   tft.setCursor(75, 75);
   tft.println("ALARME TENSION 28V");
   tft.setCursor(75, 100);
   tft.print("TENSION = ");

  afTension = (float)Tension;            // conversion Tension en float pour affichage
  afTension = afTension /100;            // valeur en V
  tft.print(afTension ,1);
  tft.print(" V");

   tft.setCursor(150, 200);
   tft.print("Reset ESP32");
   while (true) { }                      // boucle infinie	
  }
 else if (Tension < TensionMIN28 )
  {
   digitalWrite(ALIM13V_ON, LOW);        // coupe le +13V
   delay(3);
   digitalWrite(BIAS_OFF, HIGH);         // coupe le BIAS
   tft.fillRect(0, 95, 99, 40, TFT_BLACK); // pour effacer le texte précédent
   tft.setCursor(112, 122);
   tft.setTextColor(TFT_RED);    // Set text color to red
   tft.print("Power Supply 28V OFF ");
   tft.setTextColor(TFT_WHITE); // Set text color to white

   detachInterrupt(digitalPinToInterrupt(26)); // bloque le PTT
  }
 else
  {
 if (abs(LectTension28 - LectTension28old) > 22 )  // si changement > 0.2V
   { LectTension28old = LectTension28;
    afTension = (float)Tension;          // conversion Tension en float pour affichage
    afTension = afTension /100;          // valeur en V

    tft.fillRect(0, 95, 99, 40, TFT_BLACK); // pour effacer le texte précédent
    tft.setCursor(5, 125);
    tft.print(afTension ,1);
    tft.print(" V");
  attachInterrupt(digitalPinToInterrupt(26), Inter_PTT,CHANGE); // init interruption sur GPIO 26
   }
  }

//---------------------------- Affichage Tension ALIM 13V -------------------------
   LectTension13 = analogRead(analogV13);       // 0 a 4095

 if (abs(LectTension13 - LectTension13old) > 42 )  // si changement > 0.2V
{ LectTension13old = LectTension13;
  Tension = LectTension13 * VMAX13 / 4095; // calcul tension 13V en dizaines de mV

 if (Tension > TensionMAX13 )            // si Tension > seuil Alarme
  {
   digitalWrite(ALIM13V_ON, LOW);        // coupe le +13V
   delay(3);
   digitalWrite(BIAS_OFF, HIGH);         // coupe le BIAS
   delay(3);
   digitalWrite(ALIM28V_OFF, HIGH);      // coupe le +28V

   detachInterrupt(digitalPinToInterrupt(26)); // bloque le PTT

   tft.fillScreen(TFT_BLACK);            // efface ecran
   tft.setFreeFont(&FreeSansBold12pt7b); // police de caractère
   tft.setTextColor(TFT_RED, TFT_BLACK); // Set text color to red on black
   tft.setCursor(75, 75);
   tft.println("ALARME TENSION 13V");
   tft.setCursor(75, 100);
   tft.print("TENSION = ");

  afTension = (float)Tension;            // conversion Tension en float pour affichage
  afTension = afTension /100;            // valeur en V
  tft.print(afTension ,1);
  tft.print(" V");

   tft.setCursor(150, 200);
   tft.print("Reset ESP32");
   while (true) { }                      // boucle infinie	
  }

  afTension = (float)Tension;                 // conversion Tension en float
  afTension = afTension /100;                 // valeur en V

  tft.fillRect(0, 155, 99, 40, TFT_BLACK);    // pour effacer le texte précédent
  tft.setCursor(5, 185);
  tft.print(afTension ,1);
  tft.print(" V");
}

//---------------------------- Affichage Courant ALIM 28V -------------------------
   LectI28 = analogRead(analogI28);           // HALF28 a 4095

 if ( LectI28 < HALF28 ) {LectI28 = HALF28;}  //
   Intensite = (LectI28 - HALF28) * 13;       // calcul intensite en mA (0 a 25000)

 if (Intensite > IntensMAX28 )           // si Intensite > seuil Alarme
  {
   digitalWrite(ALIM13V_ON, LOW);        // coupe le +13V
   delay(3);
   digitalWrite(BIAS_OFF, HIGH);         // coupe le BIAS
   delay(3);
   digitalWrite(ALIM28V_OFF, HIGH);      // coupe le +28V

   detachInterrupt(digitalPinToInterrupt(26)); // bloque le PTT

   tft.fillScreen(TFT_BLACK);            // efface ecran
   tft.setFreeFont(&FreeSansBold12pt7b); // police de caractère
   tft.setTextColor(TFT_RED, TFT_BLACK); // Set text color to red on black
   tft.setCursor(75, 75);
   tft.println("ALARME INTENSITE 28V");
   tft.setCursor(75, 100);
   tft.print("INTENSITE = ");

   afIntensite = (float)Intensite_max28;        // conversion Intensite en float pour affichage
   tft.print (afIntensite/1000 ,1);
   tft.print(" A");

   tft.setCursor(150, 200);
   tft.print("Reset ESP32");
   while (true) { }           // boucle infinie	
  }

   if (Intensite >= Intensite_max28)
    {lastTiMAX28 = millis();
     Intensite_max28 = Intensite;}            // memorise intensite max

   if (millis() > (lastTiMAX28 + T_pepHOLD))  // clear the peak after hold time
      { Intensite_max28 = Intensite; }

 if (abs(Intensite_max28 - Intensite_max28old) > 100 ) // si changement > 100mA
{  Intensite_max28old = Intensite_max28;
   largeur = Intensite_max28 / 50;            // 0 a 280 pour 14A
   if (largeur > 278) {largeur = 278;}

  tft.fillRect(101, 96, 278, 38, TFT_BLACK);  // pour effacer le rectangle précédent
  tft.fillRect(101, 96, largeur, 38, TFT_GREEN);
   if (largeur > 200)    // si > 10A
    {tft.fillRect(301, 96, (largeur - 200), 38, TFT_ORANGE);}
   if (largeur > 250)    // si > 12.5A
    {tft.fillRect(351, 96, (largeur - 250), 38, TFT_RED);}

  afIntensite = (float)Intensite_max28;       // conversion Intensite en float pour affichage 
  tft.fillRect(390, 95, 90, 40, TFT_BLACK);   // pour effacer le texte précédent
  tft.setCursor(390, 125);
  tft.print(afIntensite/1000, 1);
  tft.print(" A");
}

//---------------------------- Affichage Courant ALIM 13V -------------------------
   LectI13 = analogRead(analogI13);           // HALF13 a 4095

 if ( LectI13 < HALF13 ) {LectI13 = HALF13;}  //
   Intensite = (LectI13 - HALF13) * 12;       // calcul intensite en mA (0 a 25000)

   if (Intensite >= Intensite_max13)
    {lastTiMAX13 = millis();
     Intensite_max13 = Intensite;}            // memorise intensite max

   if (millis() > (lastTiMAX13 + T_pepHOLD))  // clear the peak after hold time
      { Intensite_max13 = Intensite; }

 if (abs(Intensite_max13 - Intensite_max13old) > 100 ) // si changement > 100 mA
{  Intensite_max13old = Intensite_max13;
   largeur = Intensite_max13 / 10;            // 0 a 280 pour 2.8A
   if (largeur > 278) {largeur = 278;}

  tft.fillRect(101, 156, 278, 38, TFT_BLACK); // pour effacer le rectangle précédent
  tft.fillRect(101, 156, largeur, 38, TFT_GREEN);
   if (largeur > 200)    // si > 2A
    {tft.fillRect(301, 156, (largeur - 200), 38, TFT_ORANGE);}
   if (largeur > 250)    // si > 2.5A
    {tft.fillRect(351, 156, (largeur - 250), 38, TFT_RED);}

  afIntensite = (float)Intensite_max13;       // conversion Intensite en float pour affichage
  tft.fillRect(390, 155, 90, 40, TFT_BLACK);  // pour effacer le texte précédent
  tft.setCursor(390, 185);
  tft.print(afIntensite/1000, 1);
  tft.print(" A");
}

//----------------------------- Gestion temperature -------------------------------
  board_temp = get_12bit_reg(0x00);
  temperature = twocomplement_temp(board_temp);
  Celsius = int (temperature);
  if (abs(Celsius - oldCelsius) >= 1 )          // si changement >= 1
//  if (Celsius != oldCelsius)                  // si changement
  { oldCelsius = Celsius;
    largeur = Celsius * 3;                      // 0 a 280 (93°)
    tft.fillRect(101, 216, 278, 38, TFT_BLACK); // pour effacer le rectangle précédent
    tft.fillRect(101, 216, largeur, 38, TFT_GREEN);

   if (largeur > 135)    // si > 45°
    {tft.fillRect(235, 216, (largeur - 135), 38, TFT_ORANGE);}
   if (largeur > 150)    // si > 50°
    {tft.fillRect(250, 216, (largeur - 150), 38, TFT_RED);}

   tft.fillRect(390, 215, 90, 40, TFT_BLACK);   // pour effacer le texte précédent
   tft.setCursor(390, 245);
   tft.print(Celsius);
   tft.drawCircle(430, 235, 5, TFT_WHITE);
   tft.setCursor(440, 245);
   tft.print("C");
  }

if (Celsius > Tempmax2)                  // si Temperature > seuil Alarme
  {
   digitalWrite(ALIM13V_ON, LOW);        // coupe le +13V
   delay(3);
   digitalWrite(BIAS_OFF, HIGH);         // coupe le BIAS
   delay(3);
   digitalWrite(ALIM28V_OFF, HIGH);      // coupe le +28V

   detachInterrupt(digitalPinToInterrupt(26)); // bloque le PTT

   tft.fillScreen(TFT_BLACK);            // efface ecran
   tft.setFreeFont(&FreeSansBold12pt7b); // police de caractère
   tft.setTextColor(TFT_RED, TFT_BLACK); // Set text color to red on black
   tft.setCursor(75, 75);
   tft.println("ALARME TEMPERATURE");
   tft.setCursor(75, 100);
   tft.print("TEMP = ");

   tft.print(Celsius);
//   tft.write(0xf8); // °
   tft.print(" C");

   tft.setCursor(150, 200);
   tft.print("Reset ESP32");
   while (true) { }                        // boucle infinie	
  }

   if (digitalRead(transmit) == LOW)       // si emission
    {
      if (Celsius > TempminTx)             // et temperature > TempminTX : FAN ON
       { digitalWrite(FAN, HIGH);          // ventilateur ON
if (flagFAN == true)
      {flagFAN = false;
         tft.fillRect(340, 280, 130, 40, TFT_ORANGE); // pour effacer le texte précédent
         tft.setCursor(355, 308);
         tft.print("FAN ON ");
      }
       }

       if (millis() > (lastTemps1 + 1000)) // chaque seconde
        { lastTemps1 = millis();           // reinitialise lastTemps1
            ++secondes;
            if (secondes > 59)
            {secondes = 0;
            ++minutes;}
            tft.fillRect(110, 260, 130, 60, TFT_BLACK); // pour effacer le texte précédent
            tft.setCursor(150, 310);

            if (secondes < 10) tft.print(" ");
            tft.print(secondes);
            tft.print(char(34));           // "

            tft.setCursor(110, 310);
            tft.print(minutes);
            tft.print(char(39));           // '
        }

if (flag4 == true) 
      {  flag4 = false;
         tft.fillRect(5, 280, 80, 40, TFT_RED);   // pour effacer le texte précédent
         tft.setCursor(25, 308);
         tft.print( "Tx");
      }
         flag5 = true;
         flag6 = true;
    }
   else
    {                                  // si reception
       if (Celsius < TempminRx )       // et temperature < TempminRx : FAN OFF
        {
if (flag5 == true) 
      {  flag5 = false;
         digitalWrite(FAN, LOW);
         tft.fillRect(340, 280, 130, 40, TFT_DARKCYAN); // pour effacer le texte précédent
         tft.setCursor(355, 308);
         tft.print( "FAN OFF");
      }
        }

if (flag6 == true) 
      {  flag6 = false;
         tft.fillRect(5, 280, 80, 40, TFT_DARKCYAN);   // pour effacer le texte précédent
         tft.fillRect(100, 280, 130, 40, TFT_BLACK);   // pour effacer le texte précédent
         tft.setCursor(25, 308);
         tft.print( "Rx");
      }
         flag4 = true;
         flagFAN = true;
    }
 }
}
//-------------------- Fin de boucle principale ----------------------------------

// routine qui redessine le curseur1 ( transistor PEAK )
void dessineCurseur1(uint16_t posx1)
{
  uint16_t postxt1;
  // on ne permet pas que le bouton dépasse les limites établies
  if (posx1 < LIMITE_GAUCHE) {posx1 = LIMITE_GAUCHE;} 
  if (posx1 > LIMITE_DROITE) {posx1 = LIMITE_DROITE;}

  // on efface le bouton, là où il se trouvait précédemment
  tft.fillRect(x_precedent1 - LARGEUR / 2, POSY1 - HAUTEUR / 2 - 2 , LARGEUR, HAUTEUR + 4, TFT_BLACK);
  // on dessine la ligne horizontale
  tft.drawLine(LIMITE_GAUCHE, POSY1, LIMITE_DROITE, POSY1, TFT_WHITE);
  // on dessine le bouton à sa nouvelle position
  tft.fillRect(posx1 - LARGEUR / 2, POSY1 - HAUTEUR / 2, LARGEUR, HAUTEUR, TFT_GREEN);

  dac_base = calcule_valeur(posx1);
//  Serial.println(dac_base);   // envoi la valeur sur la liason serie

  write_dac_base(0x68, dac_base);
  texte_dac_basePeak ();

  x_precedent1 = posx1;
}

// routine qui redessine le curseur2 ( transistor MAIN )
void dessineCurseur2(uint16_t posx2)
{
  uint16_t postxt2;
  // on ne permet pas que le bouton dépasse les limites établies
  if (posx2 < LIMITE_GAUCHE) {posx2 = LIMITE_GAUCHE;} 
  if (posx2 > LIMITE_DROITE) {posx2 = LIMITE_DROITE;}

  // on efface le bouton, là où il se trouvait précédemment
  tft.fillRect(x_precedent2 - LARGEUR / 2, POSY2 - HAUTEUR / 2 - 2 , LARGEUR, HAUTEUR + 4, TFT_BLACK);
  // on dessine la ligne horizontale
  tft.drawLine(LIMITE_GAUCHE, POSY2, LIMITE_DROITE, POSY2, TFT_WHITE);
  // on dessine le bouton à sa nouvelle position
  tft.fillRect(posx2 - LARGEUR / 2, POSY2 - HAUTEUR / 2, LARGEUR, HAUTEUR, TFT_GREEN);

  dac_base = calcule_valeur(posx2);
//  Serial.println(dac_base);   // envoi la valeur sur la liason serie

  write_dac_base(0x6A, dac_base);
  texte_dac_baseMain ();

  x_precedent2 = posx2;
}

// routine qui calcule la valeur numérique (900 à 1800) qui correspond à la position du bouton coulissant
  uint16_t calcule_valeur (uint16_t valeur_brute) {
  uint16_t valeur_convertie;
//  valeur_convertie = ( 1.0 * (valeur_brute - LIMITE_GAUCHE) / ( LIMITE_DROITE - LIMITE_GAUCHE) * VALEUR_MAX) + 900;
  valeur_convertie = ((valeur_brute - LIMITE_GAUCHE) * 2) + 900;
  return valeur_convertie;
}

void texte_dac_basePeak ()
{
  tft.fillRect(0, 40, 480, 25, TFT_BLACK); // pour effacer le texte précédent
  tft.setFreeFont(&FreeSansOblique12pt7b); // police de caractère
  tft.setTextColor(TFT_WHITE, TFT_BLACK); // Set text color to white on black
  tft.setCursor(1, 64);
  tft.print("dac_basePeak = "); // Draw text at position (5, 60)
  tft.print (dac_base);

  float VgateP = 5 * (float) dac_base /4095;
//  Serial.println(VgateP);   // envoi la valeur sur la liason serie

  tft.setCursor(255, 64);
  tft.print("VgatePeak = "); // Draw text at position (260, 60)
  tft.print (VgateP);
  tft.print ("V");
}

void texte_dac_baseMain ()
{
  tft.fillRect(0, 135, 480, 25, TFT_BLACK); // pour effacer le texte précédent
  tft.setFreeFont(&FreeSansOblique12pt7b);  // police de caractère
  tft.setTextColor(TFT_WHITE, TFT_BLACK);   // Set text color to white on black
  tft.setCursor(1, 159);
  tft.print("dac_baseMain = "); // Draw text at position (5, 160)
  tft.print (dac_base);

  float VgateM = 5 * (float) dac_base /4095;
//  Serial.println(VgateM);   // envoi la valeur sur la liason serie

  tft.setCursor(255, 159);
  tft.print("VgateMain = "); // Draw text at position (260, 160)
  tft.print (VgateM);
  tft.print ("V");
}

void dessin_rectangles_LUT ()
{
  tft.fillRect(50, 60, 175, 40, TFT_CYAN);   // rectangle largeur 175  hauteur 40
  tft.fillRect(255, 60, 175, 40, TFT_CYAN);  // rectangle largeur 175  hauteur 40
  tft.setFreeFont(&FreeSansOblique12pt7b);   // police de caractère
  tft.setTextColor(TFT_BLACK);               // Set text color to black
  tft.setCursor(53, 87);
  tft.print("Address 0x");
  tft.setCursor(258, 87);
  tft.print("Value 0x");
}
 /*****************************************************************
 * write_dac_base
 */

void write_dac_base(byte reg_base, uint16_t dac_base)
{
   dac_base_lsb = dac_base & 0x00FF;   // isole LSB
   dac_base_msb = dac_base & 0xFF00;   // isole MSB
   dac_base_msb = (dac_base_msb >> 8); // decale MSB
   dac_base_lsb = byte (dac_base_lsb);
   dac_base_msb = byte (dac_base_msb);

if (I2C_address == 0x41)           // address of LMP92066 FINAL (A0=1 A1=0)
 { write_8bits(reg_base, (dac_base_msb + 0x10)); } // bit POL = 1 courbe decroissante FINAL
 else if ( I2C_address == 0x3F)    // address of LMP92066 DRIVER (A0=0 A1=0)
 { write_8bits(reg_base, dac_base_msb); }          // bit POL = 0 courbe croissante DRIVER
   write_8bits((reg_base + 1), dac_base_lsb);
}

 /*****************************************************************
 * ecriture octet
 */

void write_8bits(byte reg_address, byte val)
 {
  Wire.beginTransmission(I2C_address);
  Wire.write(reg_address); // adresse
  Wire.write(val);         // data
  Wire.endTransmission();
 }

 /*****************************************************************
 * set acces level for write
 */
 int set_acc_lvl_L2() // for now always level 2 (3)
 {
  byte test;

  Wire.beginTransmission(I2C_address);
  Wire.write(0x11); //access control register address
  Wire.write(0xCD); //password, first word
  Wire.endTransmission();  

  Wire.beginTransmission(I2C_address);
  Wire.write(0x11); //access control register address
  Wire.write(0xF0); //password, second word, level 2
  Wire.endTransmission();

  //read back
  Wire.beginTransmission(I2C_address);
  Wire.write(0x11); //access control register address
  Wire.endTransmission(false);// don't drop connection yet  
  Wire.requestFrom(I2C_address, (uint8_t) 1);//drop connection after reading one byte
  if (Wire.available())    // slave may send less than requested
    {
      test = Wire.read();
      return test;
    }
  else
    return -1;
 }

/*****************************************************************
 * get value of an 8-bit register
 */
byte get_register(byte reg_address)
{
  byte reg;
  Wire.beginTransmission(I2C_address);
  Wire.write(reg_address); //starting register
  Wire.endTransmission(false);// don't drop connection
  Wire.requestFrom(I2C_address, (uint8_t) 1);//drop connection after reading one byte
  
  if (Wire.available())    // slave may send less than requested
    {
      reg = Wire.read();    // receive byte
    }
  else reg=0;
  return reg;
}

/*****************************************************************
 * get 12-bit value from 2 registers, MSB first
 */
int get_12bit_reg(byte reg_address)
{
  int temp=0;

  Wire.beginTransmission(I2C_address);
  Wire.write(reg_address); //starting register
  Wire.endTransmission(false);// don't drop connection
  Wire.requestFrom(I2C_address, (uint8_t) 2);//drop connection after reading two bytes
  
  if (Wire.available())    // slave may send less than requested
    {
      byte msb = Wire.read();    // receive first (MSB) byte
//      tempstore = msb & 0xF0; // store upper nibble
      if (Wire.available())
        {
          byte lsb = Wire.read(); // then LSB
          temp=msb & 0x0F;        // remove 4 most significant bits
          temp=(temp<<8)+lsb;     // assemble 12-bits to integer
        }  
      else temp=0;
    }
  else temp=0;
  
  return temp;
}

  /*****************************************************************
  *  write 12-bits to two-byte register, make sure upper nibble
  *  of MSB is correct as it will also be overwritten
  */

int write_reg_12bits(byte reg_address, unsigned int temp)
 {
  byte txbyte;
  Wire.beginTransmission(I2C_address);
  Wire.write(reg_address); //starting register
  txbyte = temp>>8; // MSB first
  Wire.write(txbyte);
  txbyte = temp & 0xFF; //LSB byte
  Wire.write(txbyte);
  Wire.endTransmission();// drop connection
 }

/*****************************************************************
 * convert 12-bit two's complement number to temperature in base10
 */
float twocomplement_temp(int temp)
{
  //converts 2-complement 12-bits to signed integer (int)
  //and converts that to degrees Celcius by multiplying by 0.0625 degrees/bit
  float tFloat;
  temp=temp & 0x0FFF; // use only lowest 12 bits
  if (temp>0x7FF)
    {
      temp=~temp+1;
      temp=-temp;
    }
  tFloat=temp * 0.0625; // conversion function, see datasheet
  return tFloat;
  
}

void touch_calibrate()
{
  uint16_t calData[5];
  uint8_t calDataOK = 0;

  // check file system exists
  if (!SPIFFS.begin()) {
    Serial.println("formatting file system");
    SPIFFS.format();
    SPIFFS.begin();
  }

  // check if calibration file exists and size is correct
  if (SPIFFS.exists(CALIBRATION_FILE)) {
    if (REPEAT_CAL)
    {
      // Delete if we want to re-calibrate
      SPIFFS.remove(CALIBRATION_FILE);
    }
    else
    {
      File f = SPIFFS.open(CALIBRATION_FILE, "r");
      if (f) {
        if (f.readBytes((char *)calData, 14) == 14)
          calDataOK = 1;
        f.close();
      }
    }
  }

  if (calDataOK && !REPEAT_CAL) {
    // calibration data valid
    tft.setTouch(calData);
  } else {
    // data not valid so recalibrate
    tft.fillScreen(TFT_BLACK);
    tft.setCursor(20, 0);
    tft.setTextFont(2);
    tft.setTextSize(1);
    tft.setTextColor(TFT_WHITE, TFT_BLACK);

    tft.println("Touch corners as indicated");

    tft.setTextFont(1);
    tft.println();

    if (REPEAT_CAL) {
      tft.setTextColor(TFT_RED, TFT_BLACK);
      tft.println("Set REPEAT_CAL to false to stop this running again!");
    }

    tft.calibrateTouch(calData, TFT_MAGENTA, TFT_BLACK, 15);

    tft.setTextColor(TFT_GREEN, TFT_BLACK);
    tft.println("Calibration complete!");

    // store data
    File f = SPIFFS.open(CALIBRATION_FILE, "w");
    if (f) {
      f.write((const unsigned char *)calData, 14);
      f.close();
    }
  }
}

